/*
 * *****************************************************************
 * *                                                               *
 * *    Copyright (c) Digital Equipment Corporation, 1995          *
 * *                                                               *
 * *   All Rights Reserved.  Unpublished rights  reserved  under   *
 * *   the copyright laws of the United States.                    *
 * *                                                               *
 * *   The software contained on this media  is  proprietary  to   *
 * *   and  embodies  the  confidential  technology  of  Digital   *
 * *   Equipment Corporation.  Possession, use,  duplication  or   *
 * *   dissemination of the software and media is authorized only  *
 * *   pursuant to a valid written license from Digital Equipment  *
 * *   Corporation.                                                *
 * *                                                               *
 * *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
 * *   by the U.S. Government is subject to restrictions  as  set  *
 * *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
 * *   or  in  FAR 52.227-19, as applicable.                       *
 * *                                                               *
 * *****************************************************************
 */
/*
 * HISTORY
 */

#ifndef lint
static char rcsid[] = "@(#)$RCSfile: simpleX.c,v $ $Revision: 1.1.2.2 $ (DEC) $Date: 1995/01/05 14:54:27 $";
#endif

/*
  SimpleX.c - Simple interface to X
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/timers.h>
#include <math.h>
#define TIMEOFDAY	1	/* time of day clock type */

#include "simpleX.h"
#include "dope.h"

#ifndef M_2PI
#define M_2PI (2*3.14159265358979)
#endif

#include "simpleX.h"

Display *	display;
Window		window;
GC		gc;
int		fg;
int		bg;
XImage *	image;

void getarg_(int *k, char *arg);

void
  SetupXWindow(w, h, setup_color, display_name, window_title)
int w, h;
int setup_color;
char* display_name;
char* window_title;
{
  XEvent event;
  XGCValues gcvalues;
  int mask;
  int i;

  display = XOpenDisplay((display_name ? display_name : ""));
  if (!display) {
    printf("Error opening X display %s\n", (display_name ? display_name : "<default>"));
    exit(1);
  }
  fg = BlackPixelOfScreen(DefaultScreenOfDisplay(display));
  bg = WhitePixelOfScreen(DefaultScreenOfDisplay(display));
  window = XCreateSimpleWindow(display, DefaultRootWindow(display),
			       0, 0, w, h, 2, fg, bg);
  mask = ExposureMask | ButtonPressMask | KeyPressMask;
  XSelectInput(display, window, mask);
  XStoreName(display, window, window_title);
  XMapWindow(display, window);
  for (;;) {
    XNextEvent(display, &event);
    if (event.type == Expose)
    break;
  }
  
  if (setup_color)
    ChooseColorMap(0);

  gcvalues.foreground = fg;
  gcvalues.background = bg;
  mask 		      = GCForeground | GCBackground;
  gc = XCreateGC(display, window, mask, &gcvalues);
}

static void
  figure_out_color(int which, int i, 
		   unsigned short *red, 
		   unsigned short *grn, 
		   unsigned short *blu)
{
  switch (which) {
  case 0:
  default:
    *red = 0xffffL * ((long) i * 101 %256) / 255L;
    *grn = 0xffffL * ((long) i * 151 %256) / 255L;
    *blu = 0xffffL * ((long) i * 171 %256) / 255L;
    return;
  case 1:
    if (i < 64) {
      *red = 0xffffL;
      *grn = 0xffffL * i / 64;
      *blu = 0;
    } else if (i < 128) {
      *red = 0xffffL * (128 - i) / 64;
      *grn = 0xffffL;
      *blu = 0;
    } else if (i < 192) {
      *red = 0;
      *grn = 0xffffL;
      *blu = 0xffffL * (i-128) / 64;
    } else {
      *red = 0;
      *grn = 0xffffL * (256-i) / 64;
      *blu = 0xffffL;
    }
    {
      int numer = 100 - (i%10);
      *red = ((long) *red) * numer / 100;
      *grn = ((long) *grn) * numer / 100;
      *blu = ((long) *blu) * numer / 100;
    }
    return;
  case 2:			/* IHS, I=1, H varies, S=100% */
    {
      double angle = M_2PI * i /256;
      double r = cos(angle)+.5;
      double g = cos(angle+M_2PI/3)+.5;
      double b = cos(angle-M_2PI/3)+.5;
      r = (r > 1.0 ? 1.0 : r < 0.0 ? 0.0 : r);
      g = (g > 1.0 ? 1.0 : g < 0.0 ? 0.0 : g);
      b = (b > 1.0 ? 1.0 : b < 0.0 ? 0.0 : b);
      *red = (int) (0xffffL * r);
      *grn = (int) (0xffffL * g);
      *blu = (int) (0xffffL * b);
    }
    return;
  case 9:			/* marco */
    if (i == 128)
      *red = *grn = *blu = 0x0L;
    else if (i < 128) {
#define MAGIC 75
      *grn = *blu = 0;
      *red = 0xffffL * (127-i+MAGIC) / (100+MAGIC);
      if (i&1) *blu = *red;
    } else {
      *red = *blu = 0;
      *grn = 0xffffL * (i-129+MAGIC) / (100+MAGIC);
      if (i&1) *blu = *grn;
    }
    return;
  case 99:			/* marco */
    if (i == 128)
      *red = *grn = *blu = 0xffffL;
    else if (i < 128) {
      *grn = *blu = 0;
      *red = 0xffffL * (i+42) / (127+42);
    } else {
      *red = *blu = 0;
      *grn = 0xffffL * (129-i+127+42) / (127+42);
    }
    return;
  }
}

void
  ChooseColorMap(int which)
{
    Visual visual;
    Colormap colormap;
    XColor color;
    int i;

    visual = *DefaultVisual(display, DefaultScreen(display));
    
    colormap = XCreateColormap(display, window, &visual, AllocAll);
    for (i = 0; i < 256; i += 1) {
      color.pixel = i;
      figure_out_color(which, i, &color.red, &color.green, &color.blue);
      switch (which) {
      case 2:
	break;
      default:
	switch (i) {
	case 0: 
	case 254:
	  color.red = color.green = color.blue = 0xffffL;
	  break;
	case 1:
	case 255:
	  color.red = color.green = color.blue = 0;
	  break;
	}
      }
      color.flags = DoRed | DoGreen | DoBlue;
      XStoreColor(display, colormap, &color);
    }
    XSetWindowColormap(display, window, colormap);
    XInstallColormap(display, colormap);
    XSetWindowColormap(display, window, colormap);
  }

XImage *
  Imagify(data, w, h)
char * 		data;
int 		w, h;
{
  return XCreateImage(display, DefaultVisual(display, DefaultScreen(display)),
		      8, ZPixmap, 0, data, w, h, 8, w);
}

void
  ShowXVal(x, y, val)
int x, y, val;
{
  XEvent event;
  XGCValues gcvalues;
  int mask;
  
  gcvalues.foreground = val;
  mask = GCForeground;
  XChangeGC(display, gc, mask, &gcvalues);
  XDrawPoint(display, window, gc, x, y);
}

/*
  Interface to Xlib.  Sets up window, converts array to image, displays image
*/

int   acx, acy;
double occr, occi, ocstep;

static void
  HandleXInput(cbr_ccr, cbr_cci, cbr_cstep, cx, cy, cbr_nmin, cbr_nmax, done)
double *cbr_ccr, *cbr_cci, *cbr_cstep;
int cx, cy;
int *cbr_nmin, *cbr_nmax;
int *done;
{
  XEvent event;
  int something = 0;
  int last_was_colormap_choose = 0, this_is_colormap_choose = 0;
  while ( (XEventsQueued(display, QueuedAfterReading) > 0) || !something ) {
    XFlush(display);
    XNextEvent(display, &event);
    switch (event.type) {
    case ButtonPress:
      {
	XButtonPressedEvent * bpe = (XButtonPressedEvent *) &event;
	int x = bpe->x;
	int y = bpe->y;
	unsigned int button = bpe->button;
	*cbr_ccr = *cbr_ccr + (x - cx) * *cbr_cstep;
	*cbr_cci = *cbr_cci + (y - cy) * *cbr_cstep;
	*cbr_cstep *= (button == Button1 ? 0.1 : 
		       button == Button2 ? 2.0 :
		       0.0);
	if (bpe->state & ShiftMask) {
	  /* zoom on piont without centering */
	  *cbr_ccr -= (x - cx) * *cbr_cstep;
	  *cbr_cci -= (y - cy) * *cbr_cstep;
	}
	something = 1;
      }
      break;
    case KeyPress:
      {
	XKeyEvent * kpe = (XKeyEvent *) &event;
	KeySym ks = XLookupKeysym(kpe, 0);
	int inc;

	last_was_colormap_choose = this_is_colormap_choose;
	this_is_colormap_choose = 0;
	if (last_was_colormap_choose) 
	  ChooseColorMap(ks-'0');
	else 
	  switch (ks) {
	  case 'q':
	  case 'Q':
	    *done = 1;
	    something = 1;
            break;
	  case 'c':
	  case 'C':
	    this_is_colormap_choose = 1;
	    break;
	  case '0':               /* Return to original picture */
	    *cbr_ccr = occr;
	    *cbr_cci = occi;
	    *cbr_cstep = ocstep;
	    something = 1;
	    break;
	  case '3':               /* Magnify 30% */
	    inc = (*cbr_nmax)*30/100;
	    *cbr_nmin += inc;
	    *cbr_nmax += inc;
	    something = 1;
	    break;
	  case '5':               /* Magnify 50% */
	    inc = (*cbr_nmax)*50/100;
	    *cbr_nmin += inc;
	    *cbr_nmax += inc;
	    something = 1;
	    break;
	  case '1':               /* Magnify 100% */
	    inc = (*cbr_nmax)*100/100;
	    *cbr_nmin += inc;
	    *cbr_nmax += inc;
	    something = 1;
	    break;
	  case '2':               /* Increase iteration count by 20 to show more detail */
	    *cbr_nmin += 20;
	    *cbr_nmax += 20;
	    something = 1;
	    break;
	  case '6':               /* Increase iteration count by 60 to show more detail */
	    *cbr_nmin += 60;
	    *cbr_nmax += 60;
	    something = 1;
	    break;
	  }
      }
      break;
    }
  }
}

void
  mbdisplayx_(qnca_desc_t *target)
{
  int    h,w;
  int    t0, t1;
  
  /* Query dope vector to get h, w, address of array                  */
  /* (No consistency checks here; assume 2 dimensions, unmapped, etc. */
  h = target->dim_info[0].upper_bound - target->dim_info[0].lower_bound + 1;
  w = target->dim_info[1].upper_bound - target->dim_info[1].lower_bound + 1;
  image = Imagify(target->pointer, w, h);
  XPutImage(display, window, gc, image, 0, 0, 0, 0, w, h);
}

void
  mbinputx_(double *fccr, double *fcci, double *fcstep, int *fnmin, int *fnmax, int *done)
{
    HandleXInput(fccr, fcci, fcstep, acx, acy, fnmin, fnmax, done);
}

void
  mbsetupx_(int *h, int *w, double *ccr, double *cci, double *cstep, int *nprocs)
{
  char title[100];

  sprintf(title, "High Performance Fortran Mandelbrot (%d processor Alpha Farm)", *nprocs);
  SetupXWindow(*w, *h, 1, "", title);
  acx = *w/2; acy = *h/2;
  occr = *ccr; occi = *cci; ocstep = *cstep;
}

